@@ -1,5 +1,5 @@
-"poLCA" <-
-function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
+`poLCA` <-
+function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE,probs.start=NULL) {
     starttime <- Sys.time()
     if (!na.rm) {
         mframe <- model.frame(formula,data,na.action=NULL)
@@ -17,32 +17,46 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
     K.j <- t(matrix(apply(y,2,max)))
     R <- nclass
     S <- ncol(x)
-    probs <- list()
     eflag <- FALSE
+    probs.start.ok <- TRUE
     if (R==1) {
-        for (j in 1:ncol(y)) {
-            probs[[j]] <- matrix(NA,nrow=1,ncol=K.j[j])
-            for (k in 1:K.j[j]) { probs[[j]][k] <- sum(y[,j]==k)/sum(table(y[,j])) }
-        }
+        probs <- list()
+        for (j in 1:J) { probs[[j]] <- matrix(table(y[,j])/sum(table(y[,j])),nrow=1) }
+        probs.init <- probs
         vp <- poLCA.vectorize(probs)
         P <- 1
-        rgivy <- matrix(1,nrow=N,ncol=1)
+        rgivy <- prior <- matrix(1,nrow=N,ncol=1)
         ml <- sum(log(poLCA.ylik.C(vp,y)))
+        se <- poLCA.se(y,x,probs,prior,rgivy)
         iter <- 1
         if (S>1) {
             cat("\n ALERT: covariates not allowed when nclass=1; will be ignored. \n \n")
-            S <- 0
+            S <- 1
         }
     } else {
         if (graphs) ifelse(max(K.j)==2,layout(matrix(c(1,2),2,1),heights=c(3.2,1)),layout(matrix(seq(1,(R+1)),R+1,1),heights=c(rep(2,R),1)))
-        error <- T
+        if (!is.null(probs.start)) { # error checking on user-inputted probs.start
+            if ((length(probs.start) != J) | (!is.list(probs.start))) {
+                probs.start.ok <- FALSE
+            } else {
+                if (sum(sapply(probs.start,dim)[1,]==R) != J) probs.start.ok <- FALSE
+                if (sum(sapply(probs.start,dim)[2,]==K.j) != J) probs.start.ok <- FALSE
+                if (sum(round(sapply(probs.start,rowSums),4)==1) != (R*J)) probs.start.ok <- FALSE
+            }
+        }
+        error <- TRUE; firstrun <- TRUE
+        probs <- probs.init <- probs.start
         while (error) { # error trap
-            error <- F
+            error <- FALSE
             b <- rep(0,S*(R-1))
             prior <- poLCA.updatePrior(b,x,R)
-            for (j in 1:J) { 
-                probs[[j]] <- matrix(runif(R*K.j[j]),nrow=R,ncol=K.j[j])
-                probs[[j]] <- probs[[j]]/rowSums(probs[[j]]) 
+            if ((!probs.start.ok) | (is.null(probs.start)) | (!firstrun)) {
+                probs <- list()
+                for (j in 1:J) { 
+                    probs[[j]] <- matrix(runif(R*K.j[j]),nrow=R,ncol=K.j[j])
+                    probs[[j]] <- probs[[j]]/rowSums(probs[[j]]) 
+                }
+                probs.init <- probs
             }
             vp <- poLCA.vectorize(probs)
             iter <- 1
@@ -51,25 +65,25 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
             dll <- Inf
             while ((iter <= maxiter) & (dll > tol) & (!error)) {
                 iter <- iter+1
-                rgivy <- poLCA.postClass.C(prior,vp,y)
-                vp$vecprobs <- poLCA.probHat.C(rgivy,y,vp)
+                rgivy <- poLCA.postClass.C(prior,vp,y)      # calculate posterior
+                vp$vecprobs <- poLCA.probHat.C(rgivy,y,vp)  # update probs
                 if (S>1) {
                     dd <- poLCA.dLL2dBeta.C(rgivy,prior,x)
-                    if (!is.matrix(try(solve(-dd$hess),silent=TRUE))) {
-                        error <- T
+                    if (!is.matrix(try(ginv(-dd$hess),silent=TRUE))) {
+                        error <- TRUE
                     } else {
-                        b <- b + solve(-dd$hess) %*% dd$grad
+                        b <- b + ginv(-dd$hess) %*% dd$grad # update betas
                     }
-                    prior <- poLCA.updatePrior(b,x,R)
+                    prior <- poLCA.updatePrior(b,x,R)       # update prior
                 } else {
                     prior <- matrix(colMeans(rgivy),nrow=N,ncol=R,byrow=TRUE)
                 }
                 llik[iter] <- sum(log(rowSums(prior*poLCA.ylik.C(vp,y))))
                 dll <- llik[iter]-llik[iter-1]
                 if (is.na(dll)) {
-                    error <- T
+                    error <- TRUE
                 } else if ((S>1) & (dll < -1e-7)) {
-                    error <- T
+                    error <- TRUE
                 }
                 if (graphs) {
                     if (max(K.j)==2) {
@@ -82,36 +96,48 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
                         lwd=2,cex.axis=0.9,cex.lab=0.9)
                 }
             }
-            if ((!error) & (S>1)){
-                if (sum(diag(solve(-dd$hess))<0)>0) error <- T
+            if (!error) {
+                se <- poLCA.se(y,x,poLCA.unvectorize(vp),prior,rgivy)
+                if (se$fail) error <- TRUE
+                if (S>1) { if (sum(diag(ginv(-dd$hess))<0)>0) error <- TRUE }
             }
-            P <- colMeans(rgivy)
             ml <- llik[iter]
-            if (error) eflag <- T
+            if (error) eflag <- TRUE
+            firstrun <- FALSE
         }
     }
+    P <- colMeans(rgivy)
+    probs <- poLCA.unvectorize(vp)
+    names(probs) <- names(se$probs) <- colnames(y)
     cl.pred <- apply(rgivy,1,which.max)
     npar <- (R*sum(K.j-1)) + (R-1)
     if (S>1) npar <- npar + (S*(R-1)) - (R-1)
     aic <- (-2 * ml) + (2 * npar)
     bic <- (-2 * ml) + (log(N) * npar)
-    compy <- poLCA.compress(y[(rowSums(y==0)==0),])
-    datacell <- compy$datamat
-    rownames(datacell) <- NULL
-    freq <- compy$freq
-    if (!na.rm) {
-        fit <- matrix(Nobs * (poLCA.ylik.C(vp,datacell) %*% P))
-        Chisq <- sum((freq-fit)^2/fit) + (Nobs-sum(fit))
+    if (sum(rowSums(y==0)==0)==0) { # if no rows are fully observed
+        compy <- NULL
+        datacell <- NULL
+        freq <- NULL
+        Chisq <- NA
+        Gsq <- NA
+        predcell <- NULL
     } else {
-        fit <- matrix(N * (poLCA.ylik.C(vp,datacell) %*% P))
-        Chisq <- sum((freq-fit)^2/fit) + (N-sum(fit))
+        compy <- poLCA.compress(y[(rowSums(y==0)==0),])
+        datacell <- compy$datamat
+        rownames(datacell) <- NULL
+        freq <- compy$freq
+        if (!na.rm) {
+            fit <- matrix(Nobs * (poLCA.ylik.C(vp,datacell) %*% P))
+            Chisq <- sum((freq-fit)^2/fit) + (Nobs-sum(fit))
+        } else {
+            fit <- matrix(N * (poLCA.ylik.C(vp,datacell) %*% P))
+            Chisq <- sum((freq-fit)^2/fit) + (N-sum(fit))
+        }
+        predcell <- data.frame(datacell,observed=freq,expected=round(fit,3))
+        Gsq <- 2 * sum(freq*log(freq/fit))
     }
-    predcell <- data.frame(datacell,observed=freq,expected=round(fit,3))
-    Gsq <- 2 * sum(freq*log(freq/fit))
     resid.df <- min(N,(prod(K.j)-1))-npar
     cat("Conditional item response (column) probabilities,\n by outcome variable, for each class (row) \n \n")
-    probs <- poLCA.unvectorize(vp)
-    names(probs) <- colnames(y)
     print(probs)
     cat("Estimated class population shares \n", P, "\n \n")
     cat("Predicted class memberships (by modal posterior prob.) \n",table(cl.pred)/N, "\n \n")
@@ -120,14 +146,11 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
     cat("=========================================== \n")
     if (S>1) {
         b <- matrix(b,nrow=S)
-        #se <- matrix(sqrt(diag(solve(-dd$hess))),nrow=S)
-        se <- NULL
         for (r in 2:R) {
             cat(r,"/ 1 \n")
-            #disp <- cbind(b[,(r-1)]),se[,(r-1)])
-            disp <- matrix(b[,(r-1)],ncol=1)
+            disp <- cbind(b[,(r-1)],se$b[,(r-1)])
             rownames(disp) <- colnames(x)
-            colnames(disp) <- c("coefficient")#,"standard error")
+            colnames(disp) <- c("coefficient","standard error")
             print(disp)
             cat("=========================================== \n")
         }
@@ -141,17 +164,22 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
     cat("BIC(",R,"):",bic,"\n")
     if (S==1) cat("G^2(",R,"):",Gsq,"(Likelihood ratio/deviance statistic) \n")
     cat("X^2(",R,"):",Chisq,"(Chi-square goodness of fit) \n \n")
-    if ((iter-1)==maxiter) cat("iterations finished, MAXIMUM LIKELIHOOD NOT FOUND \n \n")
+    if ((iter-1)==maxiter) cat("ALRET: iterations finished, MAXIMUM LIKELIHOOD NOT FOUND \n \n")
+    if (!probs.start.ok) cat("ALERT: error in user-specified starting values; new start values generated \n \n")
     if (npar>N) cat("ALERT: number of parameters estimated (",npar,") exceeds number of observations (",N,") \n \n")
     if (resid.df<0) cat("ALERT: negative degrees of freedom; respecify model \n \n")
     if (eflag) cat("ALERT: estimation algorithm automatically restarted with new initial values \n \n")
+    y[y==0] <- NA
     ret <- list()
+    ret$probs.start <- probs.init      # starting values of class-conditional response probabilities
     ret$y <- data.frame(y)             # outcome variables
     ret$x <- data.frame(x)             # covariates, if specified
     ret$N <- N                         # number of observations
     ret$Nobs <- Nobs                   # number of fully observed cases (if na.rm=F)
-    ret$probs <- probs                 # estimated class-conditional "response" probabilities
+    ret$probs <- probs                 # estimated class-conditional response probabilities
+    ret$probs.se <- se$probs           # standard errors of class-conditional response probabilities
     ret$P <- P                         # estimated class population shares
+    ret$P.se <- se$P                   # standard errors of class population shares
     ret$posterior <- rgivy             # NxR matrix of posterior class membership probabilities
     ret$predclass <- cl.pred           # Nx1 vector of predicted class memberships, by modal assignment
     ret$predcell <- predcell           # Table that gives observed vs. predicted cell counts
@@ -159,12 +187,14 @@ function(formula,data,nclass=2,maxiter=1000,graphs=FALSE,tol=1e-10,na.rm=TRUE) {
     ret$numiter <- iter-1              # number of iterations until reaching convergence
     if (S>1) {
         rownames(b) <- colnames(x)
-        #rownames(se) <- colnames(x)
+        rownames(se$b) <- colnames(x)
         ret$coeff <- b                 # coefficient estimates (when estimated)
-        ret$coeff.se <- se             # standard errors of coefficient estimates (when estimated)
+        ret$coeff.se <- se$b           # standard errors of coefficient estimates (when estimated)
+        ret$coeff.V <- se$var.b        # covariance matrix of coefficient estimates (when estimated)
     } else {
         ret$coeff <- NULL
         ret$coeff.se <- NULL
+        ret$coeff.V <- NULL
     }
     ret$aic <- aic                     # Akaike Information Criterion
     ret$bic <- bic                     # Schwarz-Bayesian Information Criterion
